/**
 *  This file is part of Escudo.
 *
 *  Escudo is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Escudo is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Escudo.  If not, see <http://www.gnu.org/licenses/>.
 */

 
with (EscudoLib) {
  top.Escudo.Accounts = {
    s_pattern: "SELECT account FROM account WHERE account LIKE :account ORDER BY account",
    s_all: "SELECT account, income, expense, total FROM view_accounts ORDER BY account",		
    s_min: "SELECT id FROM account WHERE account=:account",
    s_account: "SELECT * FROM account WHERE account=:account",
    d_account: "DELETE FROM account WHERE account=:account",
    i_account: "INSERT INTO account (\"account\") VALUES (:account)",

    /****************************************
     * Initialize / Shutdown
     */
    init: function() {
      addObserver("invalid", this.Observer);
      addObserver("refresh", this.Observer);
    },

    Observer : {
      observe: function(subject, topic, data) {
        var values = top.Escudo.Operations.getAllOp();
        document.getElementById("stat-frame").contentWindow.setData(values);

        let account = document.getElementById("account-nominal-stat");
        account.builder.rebuild();
      }
    },

    /**
     * @argument pattern {string} The pattern searched.
     * @returns {array} A list of accounts matching the pattern.
     */
    getAccountsMatchingPattern: function(pattern) {
      let statement = getStatement(this.s_pattern);
      statement.params.account =  "%" + pattern + "%";

      var values = [];
      try {
        while (statement.executeStep()) {
          values.push(statement.getString(0));
        }
      } finally {
        statement.reset();
      }
      return values;
    },

    /**
     * This function returns an account id.
     * @argument account {string} The account label.
     * @returns {integer} Returns the account id corresponding to this account.
     */
    getAccountId: function(account) {
      let statement = getStatement(this.s_min);
      statement.params.account = account;
      
      var id = undefined;
      try {
        while (statement.executeStep()) {
          id = statement.getInt32(0);
        }
      } finally {
        statement.reset();
      }
      return id;
    },
    
    /**
     * This function retrieve the id of an account, or create it
     * if it does not exist.
     * @argument account {string} The account
     * @returns {Integer} The id of the account
     */
    getOrCreateAccount: function(account) {
      var id = this.getAccountId(account);
      if (id == undefined) { id = this.insert(account); }
      return id;
    },

    retrieveAccount : function(account) {
      let statement = getStatement(this.s_account);
      statement.params.account = account;
      
      var values = [];
      try {
        while (statement.executeStep()) {
          values.push({
            id: statement.getString(0),
            account: statement.getString(1)
          });
        }
      } finally {
        statement.reset();
      }
      return values;
    },
    
    /**
     * This function deletes an account. It uses the account name as a key.
     * @argument account {string} The account name to remove.
     */
    remove: function(name) {
      let statement = getStatement(this.d_account);
      statement.params.account = account;

      try {
        statement.execute();
      } finally {
        statement.reset();
      }
    },
    
    insert : function(account) {
      if (this.retrieveAccount(account).length > 0) {
        alert($STRF("err_accountAlreadyExist", [account]));
        return;
      }
      
      let statement = getStatement(this.i_account);
      statement.params.account = account;

      try {
        statement.execute();
      } finally {
        statement.reset();
      }

//      notifyObserver("refresh", null);
    }
  }
  top.Escudo.Accounts.init();
}
