/**
 *  This file is part of Escudo.
 *
 *  Escudo is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Escudo is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Escudo.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
var fu = new financialUtilities();
var qifImported = fu.import(fu.Formats.QIF, STRING);
var xmlExported = fu.export(fu.Formats.XML, qifImported);
*/

var EXPORTED_SYMBOLS = ["FinancialUtilities"];

/**
 * Different type of financial calculation
 */
var Calculations = {
	/**
	 * @returns {Double} the net present value
	 */
	npv: function(flows, rate) {
		flows.sort(
			function(a, b) {
				if (a.date < b.date) return -1;
				if (a.date > b.date) return 1;
				return 0;
			}
		);

		var firstFlow = flows.shift();

		var npv = firstFlow.amount;

		for each(flow in flows) {
			var years = Math.ceil((flow.date - firstFlow.date) / 86400000) / 365 ;
			npv += flow.amount / Math.pow(1 + rate, years);
		}
		return npv;
	},

	/**
	 * Calculates the internal rate of return
	 */
	irr: function(flows) {
		var rate = 0;
		var nextRate = 0.1;
		var sumcalc = 0;
		var sumcalc2 = 0;
		var currentDate = 0;
		var denumerator = 0;
		var maxIteration = 100;
		var iteration = 0;


		while (maxIteration--)
		{
			sumcalc = 0.0;
			sumcalc2 = 0.0;
			rate = nextRate;
			//This calculate the formula
			for (var i = 0; i < flows.length; i++)
			{
				currentDate = Math.ceil((flows[i].date - flows[0].date) / 86400000) / 365;
				denumerator = Math.pow((1 + rate), currentDate);

				if (denumerator == 0)
					return rate;

				sumcalc += flows[i].amount / denumerator;
			}

			if (Math.abs(sumcalc) < 0.000001)
				return rate;

			for (var i = 0; i < flows.length; i++)
			{
				currentDate = Math.ceil((flows[i].date - flows[0].date) / 86400000) / 365;
				denumerator = Math.pow((1 + rate), currentDate);
				sumcalc2 += -currentDate * flows[i].amount * denumerator;
			}

			if (Math.abs(sumcalc2) < 0.000001)
				return 0.0;

			nextRate = rate - (sumcalc / sumcalc2);
			iteration += 1;
		}
		return rate;
	},

	/**
	 * Calculates the amount needed to obtain a specified internal rate of return
	 */
	amountToIrr: function(flows, rate) {
		flows.sort(
			function(a, b) {
				if (a.date < b.date) return -1;
				if (a.date > b.date) return 1;
				return 0;
			}
		);

		var firstFlow = flows[0];
		var lastFlow = flows[(flows.length - 1)];

		var npv = this.npv(flows, rate);
		var years = Math.ceil((lastFlow.date - firstFlow.date) / 86400000) / 365 ;
		return  -npv * Math.pow(1 + rate, years);
	}
}


/**
 * Used to import / export a file
 */
var	Files = {
	/**
	 * The different file format managed in import and export
	 */
	Formats : {
		CSV: "csv",
		XML: "xml"
	},

	/**
	 * Parse a string representing the content of a file.
	 */
	fromFile: function(format, content) {
		if (this.Formats.hasAttribute(format)) {
		} else {
		}
	},

	/**
	 *
	 */
	toFile: function() {

	}
}

var FinancialUtilities = {};
FinancialUtilities["Calculations"] = Calculations;
FinancialUtilities["Files"] = Files;
